@@ -3,19 +3,22 @@
 using namespace Rcpp;
 
 //' Fast marginal simple regresion analyses
+//' 
+//' Fast computation of simple regression slopes for each predictor represented by a column in a matrix
 //'
-//' @description Fast computation of simple regression slopes for each predictor represented by a column in a matrix
+//' Missing values (NA, Inf, NaN) are completely disregarded and pairwise complete cases are used for the analysis.
+//' 
 //' @param y A vector of outcomes.
-//' @param x A matrix of regressor variables. Must have the same number of rows as the length of y. Missing variables are not handled
+//' @param x A matrix of regressor variables. Must have the same number of rows as the length of y. 
 //' @param addintercept A logical that determines if the intercept should be included in all analyses (TRUE) or not (FALSE)
 //' @return A data frame with three variables: coefficients, stderr, and tstat that gives the slope estimate, the corresponding standard error, and their ratio for each column in x.
 //' @author Claus Ekstrom <claus@@rprimer.dk>
 //' @examples
 //' \dontrun{
 //'   // Generate 100000 predictors and 100 observations
-//'   x <- matrix(rnorm(100*100000))
+//'   x <- matrix(rnorm(100*100000), nrow=100)
 //'   y <- rnorm(100, mean=x[,1])
-//'   mfastLM_cpp(y, x)
+//'   mfastLmCpp(y, x)
 //'
 //' }
 //' @export
@@ -34,26 +37,40 @@ DataFrame mfastLmCpp(NumericVector y, NumericMatrix x, bool addintercept=true) {
   arma::mat newX;
   arma::mat x0=arma::ones<arma::mat>(n,1);
 
-  if (addintercept) {
-    df = n-2;
-  }
-
   arma::colvec rescoef = arma::zeros(k), resse = arma::zeros(k), tstat = arma::zeros(k);
   arma::colvec coef, resid, stderrest;
   double sig2;
+
+  arma::colvec indY = arma::zeros(n), indX = arma::zeros(n);
+  for (arma::uword j=0; j<n; j++) {
+    indY(j) = R_finite(Y(j));
+  }
   
   for (arma::uword i=0; i<k; i++) {
     if (addintercept) {
-      newX = join_rows(X.cols(i,i), x0);
+      newX = join_rows(X.col(i), x0);
     } else {
-      newX = X.cols(i,i);
+      newX = X.col(i);
+    }
+    // Handling missings
+    for (arma::uword j=0; j<n; j++) {
+      indX(j) = R_finite(X(j, i));
     }
 
-    coef = arma::solve(newX, Y);
+    arma::uvec index = arma::conv_to<arma::uvec>::from(find(indY % indX));
+
+    df = index.n_elem-1;
+    if (addintercept) {
+      df -= 1;
+    }
+
+
+
+    coef = arma::solve(newX.rows(index), Y(index));
     rescoef(i) = coef(0);
-    resid = Y - newX*coef;
+    resid = Y(index) - newX.rows(index)*coef;
     sig2 = arma::as_scalar(arma::trans(resid)*resid/df);
-    stderrest = arma::sqrt(sig2 * arma::diagvec( arma::inv(arma::trans(newX)*newX)) );
+    stderrest = arma::sqrt(sig2 * arma::diagvec( arma::inv(arma::trans(newX.rows(index))*newX.rows(index))) );
     resse(i) = stderrest(0);
     tstat(i) = rescoef(i)/resse(i);
   }
@@ -64,6 +81,3 @@ DataFrame mfastLmCpp(NumericVector y, NumericMatrix x, bool addintercept=true) {
 			   Rcpp::Named("tstat")=tstat
 			   );
 }
-
-
-
